// 2009 mlab.uiah.fi/paja
// Processing <-> Arduino with SimpleMessageSystem library

/*
 Protocal using SimpleMessageSystem Arduino library.
 Each string needs to end with carrige return (13 in ASCII) 
 Read analog values: r a [a/d] [pin]
 Read digital pins: r d [pin]
 Write digital pin: w d [pin] [value]
 Write analog pins: w a [pin] [value]
 */

import processing.serial.*;
import oscP5.*;
import netP5.*;
//osc stuff

OscP5 oscP5;
NetAddress pureData;
NetAddress animata;

//serial stuff
Serial myPort;
String valueStr;
String portnumStr;
float valueFloat = 0;


//colors
boolean doCalibration = false;
int calibrationPass = 10;
int calibrationPassCounter = 0;
int colorMin = 100000;
int colorMax = -1;
int colorRange = colorMin;
int lastColor;
boolean calibrationDone = false;

int state = 0;

int direction = 1;

boolean readyToGo = true;




void sendOsc(String message,int val) {
    /* in the following different ways of creating osc messages are shown by example */
    OscMessage myMessage = new OscMessage("/"+message);
    myMessage.add(val); /* add an int to the osc message */
    /* send the message */
    oscP5.send(myMessage, pureData);
}

void setup(){
  // set window size
  size(400, 300);    

  //init osc
  oscP5 = new OscP5(this,12000);
  pureData = new NetAddress("127.0.0.1",5879);
  animata =  new NetAddress("127.0.0.1",7110);
  
  // create a font with the second font available to the system:
  PFont myFont = createFont(PFont.list()[2], 14);
  textFont(myFont);

  // show avialable ports
  println(Serial.list());

  // open a serial port wexit()ith 9600 baude rate
  portnumStr = Serial.list()[1];
  myPort = new Serial(this, portnumStr, 9600);
  // buffer until carrige return (13 in ASCII)
  myPort.bufferUntil(10);  

  // setup for line for incoming value
  stroke(255);
}

void keyPressed(){
  if(key == 'c'){
    doCalibration = true;
  }
  if(key == 'r'){
    
    readyToGo = !readyToGo;
    println("ready: "+readyToGo);
  }
}

synchronized void draw(){
  // clear the screen
  background(100);
  // setup text color;
  fill(255, 255, 255);

  // send a command to read analog values
  /* myPort.write("r a a 0");
   myPort.write(13);
   
   // send a command to read analog values
   myPort.write("r a a 1");
   myPort.write(13);*/
  // Print values on the screen
  text("Serial port: " + portnumStr, 10, 50);
  text("From serial port: " + int(valueFloat), 10, 110);
  // text("To serial port: \t\t" + i, 10, 130);

  // update the line
  ellipse(width/2, height/2, valueFloat, valueFloat);
  if(valueFloat>0)
    valueFloat--;
}

synchronized void serialEvent(Serial p){
  String bufferStr = myPort.readString();
  if (bufferStr != null){
    int l = bufferStr.length();
    valueStr = bufferStr.substring(0, l-1);
    println(valueStr);
    String[] vals = valueStr.split(";");
    for(int i=0;i<vals.length;i++){
      float v = float(vals[i]);
      switch(i){
        //color
      case 0:
        if(v>0){
          lastColor = (int)v;          
        }
        break;
        //spoon
      case 1:
        if(v>0){
          // println("Spoon:"+v);
          valueFloat = v;
          sendOsc("spoon",int(v));   
          // convert the value
          //valueFloat = map(float(valueStr), 1000, 0, 0, width/2);
        }
        break;
        //spin directionr
      case 2:
      if(int(v)!=direction){
        sendOsc("direction",int(v));
        direction = int(v);
      }
        /*if(v>0){
          //forward
          sendOsc("direction",1);
        }
        else{
          //backward
          if(int(v)!=state){
          sendOsc("direction",0);
        }*/
        break;
        
        case 3:
        if(int(v)!=state){
          if(int(v)==1)
            state = int(v);
          sendOsc("stateChange",int(v));
          state = int(v);
        }
      default:
        println("Unknown message, value: "+ v);
      }
    }
  }
  
  if(!calibrationDone){
    if(doCalibration && calibrationPassCounter < calibrationPass){
      if(!calibrateColors())
        calibrationPassCounter++;
      else
        calibrationPassCounter = 0;
      println("Color calibration ongoing: min("+colorMin+"), max("+colorMax+")");
    }
    else if(doCalibration && calibrationPassCounter >= calibrationPass){
      println("Color calibration completed: min("+colorMin+"), max("+colorMax+")");
      calibrationDone = true;
      colorRange = (colorMax - colorMin)/10;
      doCalibration = false;
      calibrationPassCounter = 0;
    }
  }else{
    int col = 0;
    while(lastColor>colorMin+(col*colorRange)){
      col++;
    }
    sendOsc("setInstrument",col); 
    sendOsc("toggleNote",1);
    //calibrateColors();
    //colorRange = (colorMax - colorMin)/10;
    //println("Color: #"+col);
  }
}


boolean calibrateColors(){
  boolean changed = false;
  if(lastColor < colorMin){
    colorMin = lastColor;
    changed = true;
  }
  if(lastColor > colorMax){
    colorMax = lastColor;
    changed = true;
  }
  return changed;
}

void moveJoint(String joint, float x, float y){
  OscMessage myMessage = new OscMessage("/joint");
  myMessage.add(joint);
  myMessage.add(x);
  myMessage.add(y);/* add an int to the osc message */
  /* send the message */
  oscP5.send(myMessage, animata);
}

void animBone(String bone, float val){
  OscMessage myMessage = new OscMessage("/anibone");
  myMessage.add(bone);
  myMessage.add(val);
  /* send the message */
  oscP5.send(myMessage, animata);
}

void layerOnOff(String layer, boolean on){
  OscMessage myMessage = new OscMessage("/layeralpha");
  myMessage.add(layer);
  myMessage.add(on?1.0:0.0);
  /* send the message */
  oscP5.send(myMessage, animata);
}

void layerDim(String layer, boolean on){
  if(on){
    for(float i=0;i<=1f;i+=0.1f){
      OscMessage myMessage = new OscMessage("/layeralpha");
      myMessage.add(layer);
      myMessage.add(i);
      oscP5.send(myMessage, animata);
    }
  }else{
    for(float i=1.0;i>=0f;i-=0.1f){
      OscMessage myMessage = new OscMessage("/layeralpha");
      myMessage.add(layer);
      myMessage.add(i);
      oscP5.send(myMessage, animata);
    }
  }
  
}






